@@ -205,10 +205,10 @@ def process_metadata_file(file):
 
             _fragments.append(bytearray(fragment_data))
 
-            return _manifest, _fragments
         except ValueError:
             return _manifest, _fragments
 
+    return _manifest, _fragments
 
 # pylint: disable=missing-function-docstring
 @click.group()
@@ -299,11 +299,12 @@ def object_parser(oid, lock, unlock, export_otc,export_json, meta, inp, out, out
 
         chip_handler = optiga.Chip()
         chip_uid = chip_handler.uid
-        unique_path = '{chip}_{uid}_{data}_{time}.json'.format(chip=chip_handler.name,
-                                                          uid=chip_uid.fw_build + chip_uid.x_coord + chip_uid.y_coord,
-                                                          data=time.strftime("%Y%m%d"),
-                                                          time=time.strftime("%H%M%S")
-                                                          )
+        unique_path = '{chip}_{uid}_{data}_{time}.json'.\
+            format(chip=chip_handler.name,
+                   uid=chip_uid.fw_build + chip_uid.x_coord + chip_uid.y_coord,
+                   data=time.strftime("%Y%m%d"),
+                   time=time.strftime("%H%M%S")
+                   )
         unique_path = unique_path.replace(' ', '_').replace('(', '').replace(')', '').replace('/', '_')
         buffer = json.dumps(port.to_json(), indent=4)
         with open(unique_path, "w+") as js:
